"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var LinkResolverPlugin_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.LinkResolverPlugin = void 0;
const components_1 = require("../components");
const converter_events_1 = require("../converter-events");
const urlPrefix = /^(http|ftp)s?:\/\//;
const brackets = /\[\[([^\]]+)\]\]/g;
/**
 * A plugin that resolves `{@link Foo}` tags.
 */
let LinkResolverPlugin = LinkResolverPlugin_1 = class LinkResolverPlugin extends components_1.ConverterComponent {
    /**
     * Create a new LinkResolverPlugin instance.
     */
    initialize() {
        super.initialize();
        this.owner.on(converter_events_1.ConverterEvents.RESOLVE_END, this.onResolve, this, -300);
    }
    /**
     * Find all old style double bracket references to symbols within the given text and transform them into a link.
     *
     * @param text  The text that should be parsed.
     * @returns The text with symbol references replaced by links.
     */
    replaceBrackets(reflection, text) {
        const parts = [];
        let begin = 0;
        brackets.lastIndex = 0;
        for (const match of text.matchAll(brackets)) {
            if (begin != match.index) {
                parts.push({
                    kind: "text",
                    text: text.substring(begin, match.index),
                });
            }
            begin = match.index + match[0].length;
            const content = match[1];
            const { target, caption } = LinkResolverPlugin_1.splitLinkText(content);
            if (!urlPrefix.test(target)) {
                const targetRefl = reflection.findReflectionByName(target);
                if (targetRefl) {
                    parts.push({
                        kind: "inline-tag",
                        tag: "@link",
                        text: caption,
                        target: targetRefl,
                    });
                }
                else {
                    this.application.logger.warn("Failed to find target: " + content);
                    parts.push({
                        kind: "inline-tag",
                        tag: "@link",
                        text: content,
                    });
                }
            }
            else {
                parts.push({
                    kind: "inline-tag",
                    tag: "@link",
                    text: caption,
                    target,
                });
            }
        }
        parts.push({
            kind: "text",
            text: text.substring(begin),
        });
        return parts;
    }
    onResolve(context) {
        for (const reflection of Object.values(context.project.reflections)) {
            this.processReflection(reflection);
        }
        let warned = false;
        const warn = () => {
            if (!warned) {
                warned = true;
                this.application.logger.warn(`README: Comment [[target]] style links are deprecated and will be removed in 0.24`);
            }
        };
        if (context.project.readme) {
            context.project.readme = this.processParts(context.project, context.project.readme, warn);
        }
    }
    processReflection(reflection) {
        const comment = reflection.comment;
        if (!comment)
            return;
        let warned = false;
        const warn = () => {
            if (!warned) {
                warned = true;
                this.application.logger.warn(`${reflection.getFullName()}: Comment [[target]] style links are deprecated and will be removed in 0.24`);
            }
        };
        comment.summary = this.processParts(reflection, comment.summary, warn);
        for (const tag of comment.blockTags) {
            tag.content = this.processParts(reflection, tag.content, warn);
        }
    }
    processParts(reflection, parts, warn) {
        return parts.flatMap((part) => this.processPart(reflection, part, warn));
    }
    processPart(reflection, part, warn) {
        if (part.kind === "text" && brackets.test(part.text)) {
            warn();
            return this.replaceBrackets(reflection, part.text);
        }
        if (part.kind === "inline-tag") {
            if (part.tag === "@link" ||
                part.tag === "@linkcode" ||
                part.tag === "@linkplain") {
                const { caption, target } = LinkResolverPlugin_1.splitLinkText(part.text);
                if (urlPrefix.test(target)) {
                    part.text = caption;
                    part.target = target;
                }
                else {
                    const targetRefl = reflection.findReflectionByName(target);
                    if (targetRefl) {
                        part.text = caption;
                        part.target = targetRefl;
                    }
                }
            }
        }
        return part;
    }
    /**
     * Split the given link into text and target at first pipe or space.
     *
     * @param text  The source string that should be checked for a split character.
     * @returns An object containing the link text and target.
     */
    static splitLinkText(text) {
        let splitIndex = text.indexOf("|");
        if (splitIndex === -1) {
            splitIndex = text.search(/\s/);
        }
        if (splitIndex !== -1) {
            return {
                caption: text
                    .substring(splitIndex + 1)
                    .replace(/\n+/, " ")
                    .trim(),
                target: text.substring(0, splitIndex).trim(),
            };
        }
        else {
            return {
                caption: text,
                target: text,
            };
        }
    }
};
LinkResolverPlugin = LinkResolverPlugin_1 = __decorate([
    (0, components_1.Component)({ name: "link-resolver" })
], LinkResolverPlugin);
exports.LinkResolverPlugin = LinkResolverPlugin;
